/*
 * OpenEAS Experlog Application Server
 * Web application development and dynamic template scripting engine
 * Copyright (c) 2004 ExperLog SARL - http://www.experlog.com
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version. This program is distributed in
 * the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details. You should have
 * received a copy of the GNU General Public License along with this program;
 * if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package org.experlog.base;

import org.experlog.openeas.api.DataObject;
import org.experlog.util.AppConfiguration;
import org.experlog.db.ESPersistentHashtable;

import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import java.sql.SQLException;

import java.util.Enumeration;
import java.util.Hashtable;

/**
 * A client cookie
 */
public class ESCookie implements org.experlog.util.Cookie, DataObject {

  private static final long serialVersionUID = 645936686936756797L;
  boolean usecookies_ = false;
  int maxage_ = -1;
  boolean persist_ = false;
  String persistDs_ = null;
  ESPersistentHashtable info_ = null;
  transient OpenClient session_ = null;
  protected double lastmodified_ = Double.POSITIVE_INFINITY;
  boolean exchangeSession_ = false; // exchange session with other apps

  public ESCookie(OpenClient session, HttpServletRequest req,
  boolean usecookies, String persistDs, int maxage) throws Exception {

    exchangeSession_ = (session != null
      && "true".equalsIgnoreCase(session.getAppConfiguration().get("app.exchangesession")));
    persist_ = (persistDs != null);
    persistDs_ = ("main".equalsIgnoreCase(persistDs) ? null : persistDs);
    info_ = new ESPersistentHashtable((persist_ ? session : null), persistDs_, null, null);
    
    setSession(session);

    setCookie(req);
    usecookies_ = usecookies;
    maxage_ = maxage;
  }
  
  protected void setSession(OpenClient session) throws SQLException {
    session_ = session;
    if(session != null) set("S_", session.getClientId(), false);
  }
  protected void setLastModified(double date) { lastmodified_ = date; }

  public void setCookie(HttpServletRequest req) throws SQLException {
    if(req != null) setCookie(new ESServletRequest(req, null, null));
  }
  

  public void setCookie(ESServletRequest req) throws SQLException {
    if(req == null) return;
    Enumeration names = req.getParameterNames();
    while (names.hasMoreElements()) {
      String name = (String) names.nextElement();
      if (name.endsWith("_")) {
        if (!name.equals("S_")) {
          String val = req.getParameter(name);
          if (val == null) val = "";
          info_.put(name, val);
          if (name.equals("A_")) info_.put("AdminId", val);
          if (name.equals("AP_")) info_.put("AdminPwd", val);
          if (name.equals("L_")) info_.put("Lang", val);
          //TBD if(name.equals("S_")) info_.put("Store", val);
          if (name.equals("W_")) info_.put("CustId", val);

          // Affiliate (Mileage) program
          if (name.equals("ML_")) info_.put("AffLogged", val);
          if (name.equals("M_")) info_.put("AffId", val);
        }
      }
    }
  }

  public String get(String name) throws SQLException {
    Object obj = info_.get(name);
    if (obj != null)
      return obj.toString();

    return null;
  }

  public void set(String name, Object val) throws SQLException {
    set(name, val, true); // persist if required
  }

  private void set(String name, Object val, boolean persist) throws SQLException {
    if(persist) info_.put(name, val);
    else info_.putNoPersist(name, val);
  }

  /**
   * For use upon end of session
   * Do NOT persist the cookie when cleared in that situation !
   */
  public void clear() {
    if(info_ != null) info_.clear();
  }

  protected void finalize() {
    if (info_ != null) info_.clear();
    info_ = null;
  }

  /**
   * Exchange method (from this class to HttpSession)
   */
  public void toHttpSession(HttpSession s) throws SQLException {
    if(s == null) return;
    Enumeration keys = info_.keys();
    while(keys.hasMoreElements()) {
      String key = keys.nextElement().toString();
      try {
        if(exchangeSession_ || "S_".equals(key)) s.setAttribute(key, info_.get(key));
      } catch(IllegalStateException e) { return; } // invalidated session !
    }
  }

  /**
   * Exchange method (from HttpSession to this class)
   */
  public void fromHttpSession(HttpSession s,
  AppConfiguration cnf, HttpServletRequest req, HttpServletResponse rsp) throws SQLException {
    
    if(s == null) return;

    Enumeration keys;
    try { keys = s.getAttributeNames(); } catch(IllegalStateException e) { return; }
    while(exchangeSession_ && keys.hasMoreElements()) {
      String key = keys.nextElement().toString();
      if(!key.equals("S_")) {
        set(key, s.getAttribute(key).toString(), false);
        if(usecookies_ && rsp != null && cnf.storeInCookie(key)) {
          Cookie c = new Cookie(key, s.getAttribute(key).toString());
          c.setMaxAge(maxage_);
          rsp.addCookie(c);
        }
      }
    }
    if(usecookies_ && req != null) {
      Cookie c[] = req.getCookies();
      for(int i=0; c != null && i<c.length; i++) {
        String val = c[i].getValue();
        // If the value contains white spaces, it is enclosed in double quotes
        // Remove the unwanted quotes, then.
        if(val.startsWith("\"") && val.endsWith("\"")
          && (val.indexOf(' ')>0 || val.indexOf('\t')>0
             || val.indexOf('\n')>0 || val.indexOf('\r')>0)) {
          val = val.substring(1, val.length()-1);
        }
        set(c[i].getName(), val, false);
      }
    }
  }

  /**
   * MBean method
   * get all attributes
   */
  public Hashtable getInfo() {
    return info_.toHashtable();
  } // method getInfo

  /**
   * MBean method
   * set all attributes
   */
  public void setInfo(Hashtable info) {
    try {
      info_.fromHashtable(info);
    } catch(SQLException ignore) { }
  } // method setInfo

  public void setObject(String name, Object obj) throws SQLException {
    if(obj != null) info_.put(name, obj);
  }
     
  public Object getObject(String name) throws SQLException {
    if (name != null)
       return info_.get(name);
    else return null;
  }
     
  public void removeObject(String name) throws SQLException {
    if (name != null) info_.remove(name);
  }

}

