/*
 * OpenEAS Experlog Application Server
 * Web application development and dynamic template scripting engine
 * Copyright (c) 2004 ExperLog SARL - http://www.experlog.com
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version. This program is distributed in
 * the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details. You should have
 * received a copy of the GNU General Public License along with this program;
 * if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package org.experlog.db;

import java.sql.DatabaseMetaData;
import java.sql.Types;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.ResultSetMetaData;
import java.util.Hashtable;
import java.util.Enumeration;
import java.util.StringTokenizer;

import org.experlog.base.OpenClient;
import org.experlog.base.ESServletRequest;
import org.experlog.openeas.api.DataObject;
import org.experlog.util.ESDbMapper;

class ESColumn {
  
  String val_;
  boolean isnull_ = false;
  int type_;
  boolean rdonly_ = false;
  boolean changed_ = false;

  public ESColumn(String val, int type, boolean writable) {
      val_ = val;
      if (val_ == null) {
          val_ = new String("");
          isnull_ = true;
      }
      type_ = type;
      rdonly_ = !writable;
  }

  public static boolean isNumeric(int type) {
      return type == Types.BIGINT
          || type == Types.DECIMAL
          || type == Types.DOUBLE
          || type == Types.FLOAT
          || type == Types.INTEGER
          || type == Types.NUMERIC
          || type == Types.REAL
          || type == Types.SMALLINT
          || type == Types.TINYINT;
  }
  
  public boolean isNumeric() {
      return ESColumn.isNumeric(type_);
  }

  public void setWritable(boolean w) {
      rdonly_ = !w;
  }

  public boolean isWritable() {
      //    return !rdonly_;
      return true;
      //TBD... useless function ? postgresql always returns false!
  }

  public void set(String val) {
      if (val_ != null && val != null && val.equals(val_))
          return; // No change
      isnull_ = (val == null);
      val_ = (isnull_ ? "" : val);
      changed_ = true;
  }

  public boolean valueChanged() {
      return changed_;
  }

  public boolean isNull() { return isnull_; }
  
  public String getString() {
      return val_;
  }

  public String toString() {
      return (val_ == null ? "null" : val_);
  }
}

public class ESDbObject implements DataObject {

    protected OpenClient session_ = null;
    String origtable_ = null;
    String dbtable_ = null;
    boolean changed_ = false;
    boolean sqlencode_ = true;
    Hashtable tuple_ = new Hashtable();
    boolean casesensitive_ = false;
    String datasource_ = null; // default main DS

    protected ESDbMapper map_ = null;

    public ESDbObject() { }

    public ESDbObject(String table, String qry, OpenClient session)
    throws SQLException {
      this(table, qry, session, null);
    }
    
    public ESDbObject(String table, String qry, OpenClient session, String dsn)
    throws SQLException {

      if (session == null || table == null || qry == null)
        throw new SQLException("ESDbObject: init error - null parameter");
      
      datasource_ = dsn;

      ResultSet rs = null;
      Connection c = null;
      Statement st = null;
      try {
        c = session.getConnection(datasource_);
        st = c.createStatement();
        session.logRequest(null, qry);
        rs = st.executeQuery(qry);
        if (!rs.next()) 
          throw new SQLException("ESDbObject: init error - object not found in database");
        String cs =
          session.getAppConfiguration().get("jdbc.CaseSensitive");
        if (cs != null)
          casesensitive_ = cs.trim().equalsIgnoreCase("true");
        //initialize(session.getJdbcConnection(), table, rs);
        try {

          initialize(c, table, rs);

        } catch (Exception e) {
        }
      } catch (Exception e) {
        throw new SQLException(e.getMessage());
      } finally {
        if (rs != null)
          try {
            rs.close();
          } catch (SQLException e) {
          }
          if (st != null)
            st.close();
          session.closeConnection(c, datasource_);
      }
    }
    
    public ESDbObject(OpenClient session, String table, ResultSet rs)
    throws SQLException {
      this(session, table, rs, null);
    }

    public ESDbObject(OpenClient session, String table, ResultSet rs, String dsn)
    throws SQLException {

      datasource_ = dsn;
      
      if (session != null) {
        map_ = session.getDbMapper();
        session_ = session;
        String cs =
          session.getAppConfiguration().get("jdbc.CaseSensitive");
        if (cs != null)
          casesensitive_ = cs.trim().equalsIgnoreCase("true");
        Connection con = null;
        try {
          con = (rs == null ? session.getConnection(datasource_) : null);
          initialize(con, table, rs);
          if(con != null) session.closeConnection(con, datasource_);
        } catch (Exception e) {
        } finally {
          // DO NOT close ResultSet, it may come from outside !
          if(con != null) session.closeConnection(con, datasource_);
        }

      } else {
        initialize(null, table, rs);
      }
    }

    String adaptCase(String s) {
      return (casesensitive_ ? s : s.toUpperCase());
    }

    public void initialize(Connection conn, String table, ResultSet rs)
    throws SQLException {

      origtable_ = table;
      if (map_ != null)
        dbtable_ = map_.getTable(origtable_);
      else
        dbtable_ = origtable_;

      String colname = null;
      int coltype = 0;

      boolean internal = (rs == null && conn != null);
      if (internal) {
        try {
          DatabaseMetaData md = conn.getMetaData();
          ResultSet rsc = md.getColumns("", "", dbtable_, "%");
          while(rsc.next()) {
            colname = rsc.getString(4);
            coltype = rsc.getInt(5);
            
            tuple_.put(
                adaptCase(colname),
                new ESColumn(
                    (ESColumn.isNumeric(coltype) ? "0" : ""),
                    coltype,
                    true)); // Assumed always writable...
          }
          rsc.close();
        } catch(Exception ee) {

          Statement st = null;
          try {
            st = conn.createStatement();
            try {
              rs = st.executeQuery("select * from " + dbtable_ + " limit 1");
            } catch(Exception rse) {
              rs = st.executeQuery("select * from " + dbtable_);
            }
            ResultSetMetaData md = rs.getMetaData();
            for (int i = 1; i <= md.getColumnCount(); i++) {
              colname = md.getColumnName(i);
              coltype = md.getColumnType(i);

              tuple_.put(
                  adaptCase(colname),
                  new ESColumn(
                      (ESColumn.isNumeric(coltype) ? "0" : ""),
                      coltype,
                      md.isWritable(i)));

            }
          } catch(Exception ignore) {
          } finally {
            if(rs != null) rs.close();
            if(st != null) st.close();
          }
        }
        
      } else if (rs != null) { // not internal
        ResultSetMetaData md = rs.getMetaData();
        for (int i = 1; i <= md.getColumnCount(); i++) {
          colname = md.getColumnName(i);
          coltype = md.getColumnType(i);
        
          tuple_.put(
              adaptCase(colname),
              new ESColumn(
                  rs.getString(i),
                  coltype,
                  md.isWritable(i)));
        }
      }

    }

    public void setSqlEncode(boolean s) {
      sqlencode_ = s;
    }

    public void setWritable(String name, boolean w) {
      if (map_ != null)
        name = map_.getField(origtable_, name);
      ESColumn col = (ESColumn) tuple_.get(adaptCase(name));
      if (col != null)
        col.setWritable(w);
    }

    private final String internal_get(String name) throws SQLException {
      if (map_ != null)
        name = map_.getField(origtable_, name);
      ESColumn col = (ESColumn) tuple_.get(adaptCase(name));
      if (col == null)
        return null;
      return col.getString();
    }

    public String get(String name) throws SQLException {
      return internal_get(name);
    }

    public void set(String name, String val) {
      if (map_ != null)
        name = map_.getField(origtable_, name);
      ESColumn col = (ESColumn) tuple_.get(adaptCase(name));
      if (col != null && col.isWritable()) {
        if (sqlencode_)
          val = sqlEncode(val);

        col.set(val);
        tuple_.put(adaptCase(name), col);
        if (col.valueChanged())
          changed_ = true;
      }
    }

    public void set(String name, int val) {
      set(name, Integer.toString(val));
    }

    public void set(String name, long val) {
      set(name, Long.toString(val));
    }

    public void set(String name, double val) {
      set(name, Double.toString(val));
    }

    public double getDouble(String name) throws SQLException {
      String v = internal_get(name);
      if (v == null)
        throw new SQLException("No such column: " + name);
      try {
        return Double.valueOf(v).doubleValue();
      } catch (NumberFormatException e) {
        throw new SQLException("Double number format exception: " + v);
      }
    }

    public boolean present(String col) {
      if (map_ != null)
        col = map_.getField(origtable_, col);
      return tuple_.get(adaptCase(col)) != null;
    }

    public Enumeration keys() { return (tuple_ == null ? null : tuple_.keys()); }
    
    public void setFromRequest(ESServletRequest req) {
      Enumeration en = req.getParameterNames();
      while (en.hasMoreElements()) {
        String pname = (String) en.nextElement();
        if (present(pname)) {
          set(pname, req.getParameter(pname));
        }
      }
    }

    public void addColumn(String name, int type, String val) {
      if (tuple_.get(adaptCase(name)) == null) {
        tuple_.put(adaptCase(name), new ESColumn(val, type, true));
      }
    }

    public void saveChanges(String where) throws SQLException {
      if (changed_ == false)
        return;
      String sql = updateQuery(where);
      if (sql != null) {
        Connection con = null;
        try {
          con = session_.getConnection(datasource_);
          Statement st = con.createStatement();
          session_.logRequest(null, sql);
          st.executeUpdate(sql);
          st.close();
          session_.closeConnection(con, datasource_);
          con = null;
        } catch (SQLException e) {
          throw e;
        } catch (Exception e) {
          throw new SQLException(e.getMessage());
        } finally {
          if(con != null) try { session_.closeConnection(con, datasource_); } catch(Exception ignore) { }
        }
      }
    }

    public void clearValues() {
      Enumeration en = tuple_.keys();
      while (en.hasMoreElements()) {
        String colname = (String) en.nextElement();
        ESColumn col = (ESColumn) tuple_.get(colname);
        if (col.isNumeric())
          set(colname, "0");
        else
          set(colname, "");
      }
    }

    //TBD the same routine is in ESDynHtml.java...
    //it should be in some sqlencoder class ?
    String sqlEncode(String s) {
      
      if(s == null) return null;
      
      String sep = null, rules = null;
      try {
        if (session_ != null)
          sep =
            session_.getAppConfiguration().get("sqlencode.Separator");
      } catch (SQLException e) {
        sep = null;
      }
      if (sep == null)
        sep = ";";

      try {
        if (session_ != null)
          rules =
            session_.getAppConfiguration().get(
            "sqlencode.Substitutions");
      } catch (SQLException e) {
        rules = null;
      }
      // Rules syntax: c1=s1;c2=s2;...cn=sn (replace char1 with string1, etc...)
      // ; is the default separator (sqlencode.separator to change it)
      // default rules: replace ' with '', and newlines with spaces.
      if (rules == null) rules = "\'=\'\';\r=\" \";\n=\" \";";

      StringTokenizer st = new StringTokenizer(rules, sep);
      Hashtable subst = new Hashtable();
      while (st.hasMoreTokens()) {
        StringTokenizer st2 = new StringTokenizer(st.nextToken(), "=");
        if (!st2.hasMoreTokens())
          continue;
        String k = st2.nextToken();
        if (!st2.hasMoreTokens())
          continue;
        String v = st2.nextToken();
        if (v.equals("\" \""))
          v = " ";
        else if (v.equals("\"\""))
          v = "";
        subst.put(k, v);
      }

      StringBuffer buf = new StringBuffer();
      for (int i = 0; i < s.length(); i++) {
        char c = s.charAt(i);
        String sub = (String) subst.get(String.valueOf(c));
        if (sub == null)
          buf.append(c);
        else
          buf.append(sub.toString());
      }

      return buf.toString();
    }

    public String toString() {
      return (tuple_ == null ? "null" : tuple_.toString());
    }

    public String deleteQuery(String where) {
      if (where == null || where.length() < 1)
        return ("DELETE FROM " + dbtable_);
      else
        return ("DELETE FROM " + dbtable_ + " WHERE " + where);
    }

    public String updateQuery(String where) {
      if (changed_ == false)
        return null;

      StringBuffer sql = new StringBuffer("UPDATE " + dbtable_ + " SET");
      Enumeration en = tuple_.keys();
      boolean first = true;
      while (en.hasMoreElements()) {
        String colname = (String) en.nextElement();
        ESColumn col = (ESColumn) tuple_.get(colname);

        if (!col.isWritable() || !col.valueChanged())
          continue;

        boolean quote = !col.isNumeric();

        sql.append(
            (first ? " " : ",")
            + colname
            + "="
            + (quote ? "\'" : "")
            + col.getString()
            + (quote ? "\'" : ""));
        first = false;
      }

      if (where != null && where.length() >= 1)
        sql.append(" WHERE " + where);

      return sql.toString();
    }

    public String insertQuery() {
      StringBuffer sql = new StringBuffer("INSERT INTO " + dbtable_ + "(");

      Enumeration en = tuple_.keys();
      boolean first = true;
      while (en.hasMoreElements()) {
        String colname = (String) en.nextElement();
        sql.append((first ? "" : ",") + colname);
        first = false;
      }
      sql.append(") VALUES (");

      en = tuple_.keys();
      first = true;
      while (en.hasMoreElements()) {
        String colname = (String) en.nextElement();
        ESColumn col = (ESColumn) tuple_.get(colname);

        boolean quote = !col.isNumeric();

        sql.append(
            (first ? "" : ",")
            + (quote ? "\'" : "")
            + col.getString()
            + (quote ? "\'" : ""));
        first = false;
      }

      sql.append(")");
      return sql.toString();
    }


    public static void main(String args[]) {
      try {
        ESDbObject dbo =
          new ESDbObject(null, "ECustomer", (ResultSet) null);
        dbo.addColumn("Id", Types.VARCHAR, "001");
        dbo.setWritable("Id", false);
        dbo.addColumn("FirstName", Types.VARCHAR, "Bill");
        dbo.addColumn("LastName", Types.VARCHAR, "Smith");
        dbo.addColumn("Age", Types.INTEGER, "30");

        dbo.set("FirstName", "John");
        System.out.println(dbo.updateQuery("Id=\'001\'"));
        System.out.println(dbo.insertQuery());
        System.out.println(dbo.deleteQuery(null));
      } catch (Exception e) {
        System.err.println("ESDbObject:main Exception " + e);
      }
    }
    
}
