/*
 * OpenEAS Experlog Application Server
 * Web application development and dynamic template scripting engine
 * Copyright (c) 2004 ExperLog SARL - http://www.experlog.com
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version. This program is distributed in
 * the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details. You should have
 * received a copy of the GNU General Public License along with this program;
 * if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package org.experlog.gencode;

import org.experlog.openeas.api.DataObject;

import java.sql.SQLException;
import java.util.Calendar;
import java.util.Date;

public class ESDateAccess implements DataObject {

  private int daysOfMonth_[] = {31,28,31,30,31,30,31,31,30,31,30,31};

	public String get(String name) throws SQLException {
		return get(name, System.currentTimeMillis());
	}

	public String get(String name, long tm) {
		name = name.trim().toUpperCase();

		if (name.equals("NOW"))
			return Long.toString(tm);

		Calendar cal = Calendar.getInstance();
		cal.setTime(new Date(tm));
		int offset;

		if (name.startsWith("YEAR")) {
			offset = getOffset(name.substring(4));
			if (offset != 0) cal.set(Calendar.YEAR, cal.get(Calendar.YEAR) + offset);
			return Long.toString(cal.getTime().getTime());

		} else if (name.startsWith("MONTH")) {
			offset = getOffset(name.substring(5));
			int curmonth = cal.get(Calendar.MONTH);
			int ny;
			if (offset >= 0) {
				// Months range 0-11
				// Ex. Oct 2004+3 months = 9+3/12 = +1 yr
				ny = (curmonth + offset) / 12;
			}
			else {
				// Months range 0-11
				// Ex: JAN 2005-12 months = 0-12-11/12 = -1 yr
				ny = (curmonth + offset - 11) / 12;
			}
			int nm;
			if (offset >= 0)
				nm = (curmonth + offset) % 12;
			else
				nm = (curmonth + offset + 12) % 12;
			if (ny != 0) cal.set(Calendar.YEAR, cal.get(Calendar.YEAR) + ny);
			// If the target month has less days than the current
                        // one, then set day to the last day of the target month
                        // (otherwise dates like Sep. 31 might appear...)
                        int day = cal.get(Calendar.DAY_OF_MONTH);
                        if(day > daysOfMonth_[nm])
                          cal.set(Calendar.DAY_OF_MONTH, daysOfMonth_[nm]);

			cal.set(Calendar.MONTH, nm); // Don't filter out 0 (Jan)
			return Long.toString(cal.getTime().getTime());

		} else if (name.startsWith("DAY")) {

			offset = getOffset(name.substring(3));
			long rdate = cal.getTime().getTime()
								 + (long) offset * 24 * 60 * 60 * 1000;

			int hour = cal.get(Calendar.HOUR_OF_DAY);
			cal.setTime(new Date(rdate));

			// Take care of daylight saving offset...
			// There may be a 1 hour shift, that can cause an error of 1 day !
			// To avoid this, add (or remove) 1 hour when crossing the daylight
			// saving day...
			int diff = cal.get(Calendar.HOUR_OF_DAY) - hour;
			if(diff > 0) rdate += 60 * 60 * 1000;  // eg. it was 0h and it is now 23h
			else if (diff < 0) rdate -= 60 * 60 * 1000; // eg. it was 23h and is it now 0h

			return Long.toString(rdate);

		} else if (name.startsWith("HOUR")) {
			offset = getOffset(name.substring(4));
			return Long.toString(cal.getTime().getTime() + (long) offset * 60 * 60 * 1000);
		}

		return null;
	}

	private int getOffset(String off) {
		int val;
		try {
			val = Integer.parseInt(off.substring(1));
		} catch (Exception e) {
			val = 0;
		}
		if (off.startsWith("-"))
			return -val;
		else
			return val;
	}

}

