/*
 * OpenEAS Experlog Application Server
 * Web application development and dynamic template scripting engine
 * Copyright (c) 2004 ExperLog SARL - http://www.experlog.com
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version. This program is distributed in
 * the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details. You should have
 * received a copy of the GNU General Public License along with this program;
 * if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 */


package org.experlog.gencode;

public class ExpEval {

  public double eval(ExpOperand formula) {
    if(formula == null) return 0;  // null evaluates to false
    else if(formula.isStringValue()) return 1; // a String evaluates to true
    else return eval2(formula);
  }

  double eval2(ExpOperand formula) {

    int operator = formula.getOperator();
    double val;
    String sval;

    switch(operator) {

      case ExpParser.VALUE: // Stop condition
        return formula.getNumericValue();

      case ExpParser.NOT:
        return (eval2(formula.singleTerm()) == 0 ? 1 : 0);

      case ExpParser.OR:
        val = eval2(formula.leftTerm()) + eval2(formula.rightTerm());
        return (val == 0 ? 0 : 1); 

      case ExpParser.AND:
        val = eval2(formula.leftTerm()) * eval2(formula.rightTerm());
        return (val == 0 ? 0 : 1); 

      case ExpParser.UMINUS:
        return - eval2(formula.singleTerm());

      case ExpParser.EXP:
        return Math.pow(eval2(formula.leftTerm()),
                eval2(formula.rightTerm()));

      case ExpParser.MULT:
        return eval2(formula.leftTerm()) * eval2(formula.rightTerm());

      case ExpParser.DIV:
        return eval2(formula.leftTerm()) / eval2(formula.rightTerm());

      case ExpParser.PLUS:
        return eval2(formula.leftTerm()) + eval2(formula.rightTerm());

      case ExpParser.MINUS:
        return eval2(formula.leftTerm()) - eval2(formula.rightTerm());

      case ExpParser.GT:
        return eval2(formula.leftTerm()) > eval2(formula.rightTerm()) ? 1 : 0;
      case ExpParser.GE:
        return eval2(formula.leftTerm()) >= eval2(formula.rightTerm()) ? 1 : 0;

      case ExpParser.LT:
        return eval2(formula.leftTerm()) < eval2(formula.rightTerm()) ? 1 : 0;
      case ExpParser.LE:
        return eval2(formula.leftTerm()) <= eval2(formula.rightTerm()) ? 1 : 0;

      case ExpParser.EQUAL:
        if(formula.leftTerm().isStringValue()
         || formula.rightTerm().isStringValue()) {
          sval = formula.leftTerm().getStringValue();
          return (sval.equals(formula.rightTerm().getStringValue()) ? 1 : 0);
        } else {
          val = eval2(formula.leftTerm());
          return (val == eval2(formula.rightTerm()) ? 1 : 0);
        }

      case ExpParser.DIFF:
        if(formula.leftTerm().isStringValue()
         || formula.rightTerm().isStringValue()) {
          sval = formula.leftTerm().getStringValue();
          return (sval.equals(formula.rightTerm().getStringValue()) ? 0 : 1);
        } else {
          val = eval2(formula.leftTerm());
          return (val == eval2(formula.rightTerm()) ? 0 : 1);
        }
    }
    return 0;
  }

}

