/*
 * OpenEAS Experlog Application Server
 * Web application development and dynamic template scripting engine
 * Copyright (c) 2004 ExperLog SARL - http://www.experlog.com
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version. This program is distributed in
 * the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details. You should have
 * received a copy of the GNU General Public License along with this program;
 * if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package org.experlog.lite;

import java.io.IOException;
import java.io.PrintWriter;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletResponse;

import org.apache.log4j.Category;
import org.experlog.base.OpenClient;
import org.experlog.base.ESServlet;
import org.experlog.base.ESServletRequest;
import org.experlog.base.ESStore;
import org.experlog.base.ESCookie;
import org.experlog.db.ESDbFormHandler;
import org.experlog.util.ESCategory;

public class OpenEAS extends ESServlet {

  private static final long serialVersionUID = -1564888231101340876L;

  public void getRequest(
    OpenClient session,
    ESServletRequest req,
    HttpServletResponse res)
    throws ServletException, IOException {
    logger = ESCategory.getCategory("openeas.servlet.OpenEAS");

    String page = null;
    try {

      String cac;
      if ((cac = req.getParameter("Cache")) != null
        && cac.equalsIgnoreCase("n"))
        setNoStore(res); // No cache

      page = req.getParameter("Page");
      if (page == null || page.length() < 1) {
        String homepage = session.getAppConfiguration().get("app.HomePage");
        if(homepage == null) // Backward compatibility
          homepage = session.getAppConfiguration().get("shop.HomePage");
        if (homepage == null)
          homepage = "index.tmpl";
        page = new String(homepage);
      }

      // Security check: page should not contain ".."
      if (!org.experlog.util.AppConfiguration.isSecurePath(page)) {
        throw new ServletException(
          "Access forbidden for security reasons " + "(wildcard in page name)");
      }

      String failurePage = req.getParameter("ActionFailed");

      if (actionFailed()) {
        if (failurePage == null) {
          throw new ServletException(
            "Action "
              + req.getParameter("Action")
              + " failed, with no ActionFailed template page defined");
        } else {
          page = failurePage;
        }
      }

      if(page.equals("this")) {
        setNoContent(res);
      } else {

        PrintWriter out = null;
        String ctyp = req.getParameter("ContentType");
        if(! "none".equalsIgnoreCase(ctyp))
          out = res.getWriter();
        else setNoContent(res);
 
        if (out != null && !session.noComment() && ctyp == null) {
          out.println(
            "<!-- Generated by openeas http://www.experlog.com -->");
        }

        ESDbFormHandler dbf = new ESDbFormHandler(session, req);
        dbf.defineDataObject("Cookie", (ESCookie) session.getCookie());
        // ShopConfig is here for backward compatibility...
        // The right name is now AppConfig
        dbf.defineDataObject("ShopConfig", session.getAppConfiguration());
        dbf.defineDataObject("AppConfig", session.getAppConfiguration());
        dbf.defineDataObject("DbMap", session.getDbMapper());
        String lang = session.getCookie().get("L_");
        if (lang != null) {
          session.getAppConfiguration().setLocale(lang);
          if(! "false".equalsIgnoreCase(session.getAppConfiguration().get("app.UseLocaleInPath")))
            dbf.displayDbForm(out, "ESTemplates/Shop/" + lang + "/" + page);
          else
            dbf.displayDbForm(out, "ESTemplates/Shop/" + page);
        } else
          dbf.displayDbForm(out, "ESTemplates/Shop/" + page);
      }

    } catch (Exception e) {
      if (session != null) {
        Category logger =
          ESCategory.getCategory(ESStore.getInstance().getShopName(), "openeas.lite");
        logger.error("ERROR IN TEMPLATE " + page + " " + e);
        e.printStackTrace();
      }
      throw new ServletException(e.getMessage());
    }
  }

  public void postRequest(
    OpenClient session,
    ESServletRequest req,
    HttpServletResponse res)
    throws ServletException, IOException {
    getRequest(session, req, res);
  }

  public String getServletInfo() {
    return "openeas servlet";
  }
}

