/*
 * OpenEAS Experlog Application Server
 * Web application development and dynamic template scripting engine
 * Copyright (c) 2004 ExperLog SARL - http://www.experlog.com
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version. This program is distributed in
 * the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details. You should have
 * received a copy of the GNU General Public License along with this program;
 * if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package org.experlog.openeas.api;

import java.sql.SQLException;

import org.experlog.base.ESAction;
import org.experlog.base.ESServletRequest;
import org.experlog.gencode.DynHtml;
import org.experlog.util.Client;

/**
 * An Expershop Action.
 * To be used by programmers (in the ExperShop API)
 */
public abstract class Action implements ESAction {

	private DynHtml page_ = null;
    
	public final boolean processRequest(Client session, ESServletRequest req, DynHtml page, Object info) throws Exception {
		page_ = page;
		return service(new Session(session), new ServletRequest(req), info);
	}
    
	public abstract boolean service(Session s, ServletRequest r, Object info)
	throws Exception;
	public abstract boolean serverSide();
    
	public final void addDataSet(String name, DataSet d) throws SQLException {
		if(page_ == null) {
	    throw new SQLException(
		   "addDataSet: current page is null - not yet initialized."
		   + " Use $Action to run this action");
		}
	
		page_.addDataSet(name, d);
	}

	public final void addDataObject(String name, DataObject d)
	throws SQLException {
		if(page_ == null) {
	    throw new SQLException(
		   "addDataObject: current page is null - not yet initialized."
		   + " Use $Action to run this action");
		}
	
		page_.addDataObject(name, d);
	}
}

