/*
 * OpenEAS Experlog Application Server
 * Web application development and dynamic template scripting engine
 * Copyright (c) 2004 ExperLog SARL - http://www.experlog.com
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version. This program is distributed in
 * the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details. You should have
 * received a copy of the GNU General Public License along with this program;
 * if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package org.experlog.util;

import org.apache.log4j.Appender;
import org.apache.log4j.Category;
import org.apache.log4j.ConsoleAppender;
import org.apache.log4j.FileAppender;
import org.apache.log4j.Layout;
import org.apache.log4j.PatternLayout;
import org.apache.log4j.Priority;

import java.util.Enumeration;
import java.util.Hashtable;

/**
 * This object defines a list of log4j categories for all the different
 * application.
 * this object is used by the store via the MBean interface to modify and
 * read the log level of an application.
 */
public class ESCategory {

	static Hashtable categories = null;
	static Hashtable levels_ = null;
	static Hashtable outputs = null;
	static String level_ = "INFO";

	/**
	 * sets the level of an application, the new level is available for all the
	 * categories for the current application.
	 * @param newLevel	the newLevel to set to categories
	 * @param application	the concerned application
	 */
	public static void setLevel(String newLevel, String application) {
		if (levels_ == null)
			levels_ = new Hashtable();

		for (Enumeration e = categories.keys(); e.hasMoreElements();) {
			String theLevel = (String) e.nextElement();
			if (theLevel.startsWith(application) || theLevel.indexOf("openeas."+application) != -1) {
//            if (theLevel.startsWith(application)) {
				Category cat = (Category) categories.get(theLevel);
				cat.setPriority(Priority.toPriority(newLevel));
			}
		}

		levels_.put(application, newLevel);
	}

	/**
	 * returns the level of the current application
	 * @param application	the application
	 * @return	the string representation of the application level.
	 */
	public static String getLevel(String application) {
		if(application == null) return level_;
		String level = (String)levels_.get(application);
		return (level == null ? level_ : level);
	}

	/**
	 * sets the output channel for a given application.
	 * @param output	the string representation of the output channel
	 * @param application	the application
	 */
	public static void setOutput(String output, String application) {
		if (outputs == null)
			outputs = new Hashtable();
		outputs.put(application, output);
	}

	public static String getOutput(String application) {
		return (String)outputs.get(application);
	}

	/**
	 * returns a Category. This object can be already defined or not.
	 * @param application	the name of the application
	 * @param cat	the name of the category.
	 * @return	a new category or an existing category
	 */
	private static Category common(String application, String cat) {
		Category category = null;
		String all = null;

		if (application == null)
			all = cat;
		else
			all = application + "." + cat;

		if (categories.get(all) == null) {

			category = Category.getInstance(all);

			Layout layout = new PatternLayout("%d [%t] (%F:%L) %-5p %c - %m%n");
			//%5p [%t] (%F:%L) - %m%n");

			Appender appender = null;
			if ((application == null) || (outputs == null)) {
				appender = new ConsoleAppender(layout, "System.err");
			} else {
				String output = (String) (outputs.get(application));
				if (output != null) {
					try {
						appender = new FileAppender(layout, output, true);
					} catch (Exception e) {
						System.err.println("Can not log into:" + output + ", an exception occurs. Uses Console instead of.");
						appender = new ConsoleAppender(layout, "System.err");
					}
				} else {
					appender = new ConsoleAppender(layout, "System.err");
				}
			}

			category.addAppender(appender);

			String theLevel;
			if (levels_ == null)
				levels_ = new Hashtable();

			if (application == null) {
				theLevel = level_;
			} else {
				try {
					theLevel = getLevel(application);
				} catch (Exception e) {
					theLevel = level_;
				}
			}

			category.setPriority(Priority.toPriority(theLevel));

			categories.put(all, category);
			return category;
		} else {
			return (Category) categories.get(all);
		}
	}

	/**
	 * returns a category to the caller object.
	 * @param application	the name of the application
	 * @param cat	the name of the category
	 * @return	an existing or a new category.
	 */
	public static Category getCategory(String application, String cat) {
		if (categories == null)
			categories = new Hashtable();
		if (levels_ == null)
			levels_ = new Hashtable();

		return common(application, cat);

	}

	/**
	 * returns a category to the caller object.
	 * @param cat	the name of the category
	 * @return	an existing or a new category.
	 */
	public static Category getCategory(String cat) {
		if (categories == null)
			categories = new Hashtable();
		if (levels_ == null)
			levels_ = new Hashtable();

		return common(null, cat);
	}
}

