/*
 * OpenEAS Experlog Application Server
 * Web application development and dynamic template scripting engine
 * Copyright (c) 2004 ExperLog SARL - http://www.experlog.com
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version. This program is distributed in
 * the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details. You should have
 * received a copy of the GNU General Public License along with this program;
 * if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 */


package org.experlog.util;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Set;
import java.util.Vector;

/**
 * This class is both a Vector and a Hashtable.
 * It warrants the order of inserted elements, and provides a direct access
 * to them using a key.
 */
public class ESHashtable {

  Hashtable h_ = null;
  Vector v_ = null;

  public ESHashtable() {
    v_ = new Vector();
    h_ = new Hashtable();
  }

  /**
   * Like Hashtable.put()
   */
  public void put(Object key, Object val) {
		Integer pos = (Integer)h_.get(key);
		if(pos == null) {
    	v_.addElement(val);
    	h_.put(key, new Integer(v_.size()-1));
		} else {
			v_.setElementAt(val, pos.intValue());
		}
  }

  /**
   * A put method to insert after a specified key
   */
  public void putAfter(Object before, Object key, Object val) {
    if(before == null || get(key) != null) {
      put(key, val);
      return;
    }

    Integer bpos = (Integer)h_.get(before);
    if(bpos == null) put(key, val);
    else insertElementAt(key, val, bpos.intValue()+1);
  }

  public int getIndex(Object key) {
    Integer idx = (Integer)h_.get(key);
    return (idx == null ? -1 : idx.intValue());
  }

  /**
   * Like Vector.insertElementAt()
   */
  public void insertElementAt(Object key, Object val, int index)
  throws ArrayIndexOutOfBoundsException {
    v_.insertElementAt(val, index);

    // Shift the whole table, starting from index
    shiftH(index, 1);

    h_.put(key, new Integer(index));
  }

  /**
   * Like Hashtable.get()
   */
  public Object get(Object key) {
    Integer pos = (Integer)h_.get(key); 
    if(pos == null) return null;
    return v_.elementAt(pos.intValue());
  }

  /**
   * Like Hashtable.remove()
   */
  public Object remove(Object key) {
    Integer pos = (Integer)h_.remove(key); 
    if(pos == null) return null;
    int at = pos.intValue();

    // Shift the whole table, starting from pos
    shiftH(at, -1);

    Object ret = v_.elementAt(at);
    v_.removeElementAt(at);
    return ret;
  }

  /**
   * Shift the whole hashtable, starting from pos
   * @param startindex The index to start from (included)
   * @param shift The shift amount (positive or negative)
   */
  private void shiftH(int startindex, int shift) {
    Enumeration e = h_.keys();
    while(e.hasMoreElements()) {
      Object k = e.nextElement();
      int p = ((Integer)h_.get(k)).intValue();
      if(p >= startindex) h_.put(k, new Integer(p+shift));
    }
  }

  /**
   * Like Hashtable.keySet()
   */
  public Set keySet() { return h_.keySet(); }

  /**
   * Like Vector.size()
   */
  public int size() { return v_.size(); }

	/**
	 * Like Hashtable.clear()
	 */
	public void clear() {
		h_.clear();
		v_.clear();
	}

  /**
   * Like Vector.elementAt()
   */
  public Object elementAt(int pos) {
    return v_.elementAt(pos);
  }

  public Enumeration elements() {
    return v_.elements();
  }

	public Hashtable toHashtable() {
		return h_;
	}

}

