/*
 * OpenEAS Experlog Application Server
 * Web application development and dynamic template scripting engine
 * Copyright (c) 2004 ExperLog SARL - http://www.experlog.com
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version. This program is distributed in
 * the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details. You should have
 * received a copy of the GNU General Public License along with this program;
 * if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package org.experlog.util;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import java.sql.SQLException;

public class ESHttpUtils {

	/**
	 * Get a parameter in an HTTP request: the parameter must be present,
	 * otherwise an SQLException is thrown.
	 */
	public static String getMandatoryParam(HttpServletRequest req,
										   String name, String msg) throws SQLException {
		String val = getParamValue(req, name);
		if (val == null) {
			throw new SQLException(name + ": " +
								   (msg == null ? "should have a value" : msg));
		}
		return val;
	}

	/**
	 * Get a parameter in an HTTP request: like HttpServletRequest's
	 * getParameter, but the trailing blanks are removed in the value.
	 */
	public static String getParamValue(HttpServletRequest req, String name) {
		String val = req.getParameter(name);
		if (val == null || val.trim().length() < 1)
			return null;
		return val.trim();
	}

	/**
	 * Extract session ID (S_) from HttpServletRequest
	 * The session ID should be in the HTTP parameters, but may also be
	 * in the HttpSession if cookies are used.
	 */
	public static String getSessionId(HttpServletRequest req) {
    String sessionid = req.getParameter("S_");
    if(sessionid == null) {
       HttpSession hs = req.getSession(false);
      if(hs != null) { 
        try {
          sessionid = (String)hs.getAttribute("S_");
        } catch(IllegalStateException e) {
          sessionid = null;
        }
      }
    }
    return sessionid;
  }
}

